const spinner = document.getElementById("progress-indicator");
const submit = document.getElementById("submit");
const message = document.getElementById("error-message");
const result_icon = document.getElementById("result-icon");

document.getElementById("registration-form").addEventListener("submit", preventSubmit);
function preventSubmit(e) {
    e.preventDefault();
}

setTimeout(update_form, 5);

function callback_click() {
    const data = new FormData(document.getElementById("registration-form"));
    if (!form_invalid(data)) {
        prepare_form_data(data);
    } else {
        console.log("Invalid form!");
    }
}

function prepare_form_data(form) {
    submit.classList.add("uk-hidden");
    spinner.classList.remove("uk-hidden");

    // we can assume the form is valid.
    var physical = form.get("modality") === "inperson";

    var basic_data = {
        timestamp: Date.now(),
        firstname: form.get("firstname"),
        lastname: form.get("lastname"),
        email: form.get("email"),
        affiliation: form.get("affiliation"),
        no_zulip: form.get("nozulip") === "on",
        presenting: form.get("presenting") === "presenting",
        level: form.get("level"),
        extra: form.get("extra"),
        physical: physical,
    }

    if (physical) {
        var carbon_data = {
            hcrw: form.get("hcrw") === "participating",
            country: form.get("departure-country"),
            city: form.get("departure-city"),
            mode: form.get("transportation"),
        };
        var post_data = { ...basic_data, ...carbon_data };
    } else {
        var post_data = basic_data;
    }

    setTimeout(() => submit_form(post_data), 500);
}

function revert_form() {
    submit.classList.remove("uk-hidden");
    spinner.classList.add("uk-hidden");
    message.classList.add("uk-hidden");
    result_icon.removeAttribute("uk-icon");
    result_icon.classList.add("uk-hidden");
    message.classList.remove("uk-text-success", "uk-text-danger");
    result_icon.classList.remove("uk-text-success", "uk-text-danger");
}

async function submit_form(data) {
    try {
        const response = await fetch("/2023/api/register", {
            method: "POST",
            mode: "cors",
            headers: {
                "Content-Type": "application/json",
            },
            body: JSON.stringify(data),
        })

        const result = await response.text();

        spinner.classList.add("uk-hidden");
        result_icon.classList.remove("uk-hidden");
        message.classList.remove("uk-hidden");
        if (result === "OK") {
            result_icon.setAttribute("uk-icon", "icon: check");
            result_icon.classList.add("uk-text-success");
            message.classList.add("uk-text-success");
            message.innerHTML = "Success, you should receive a confirmation email shortly...";

            setTimeout(() => document.location.href = "/2023/", 2000);
        } else {
            result_icon.setAttribute("uk-icon", "icon: bolt");
            result_icon.classList.add("uk-text-danger");
            message.classList.add("uk-text-danger");
            message.innerHTML = result;

            setTimeout(revert_form, 2000);
        }
    } catch (e) {
        revert_form();
    }
}

async function fetch_cities(country) {
    const response = await fetch("/2023/api/cities/" + country, {
        method: "GET",
        mode: "cors",
    });
    return response;
}

function update_cities(country) {
    const city_input = document.getElementById("departure-city");
    city_input.value = "loading cities in " + country + "...";

    fetch_cities(country).then((response) => {
        response.text().then((response) => {
            const arr = response.split("|");
            console.log(arr);
            let city_option = document.getElementById("cities");

            city_option.innerHTML = "";

            for (city_index in arr) {
                var option = document.createElement("option");
                option.value = arr[city_index];
                city_option.appendChild(option);
            }

            city_input.value = "";
            city_input.disabled = false;
        })
    })
}

function modality_toggle(online) {
    const carbon_questions = document.getElementById("carbon-questions");
    carbon_questions.disabled = online;
    carbon_questions.hidden = online;
}

function toggle_class(element, name, toggle) {
    if (toggle) {
        element.classList.add(name);
    } else {
        element.classList.remove(name);
    }
}

function form_invalid(form) {
    const mailRegex = /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)+$/;
    const email_invalid = !form.get("email").match(mailRegex);
    const presenting_invalid = form.get("presenting") === null;
    const level_invalid = form.get("level") === null;
    const lastname_invalid = form.get("lastname") === null || form.get("lastname") === "";
    const firstname_invalid = form.get("firstname") === null || form.get("firstname") === "";
    const modality_invalid = form.get("modality") === null;
    const affiliation_invalid = form.get("affiliation") === null || form.get("affiliation") === "";
    const hcrw_invalid = form.get("hcrw") === null;
    toggle_class(document.getElementById("level-label"), "notvalid", level_invalid);
    toggle_class(document.getElementById("presenting-label"), "notvalid", presenting_invalid);
    toggle_class(document.getElementById("email-label"), "notvalid", email_invalid);
    toggle_class(document.getElementById("firstname-label"), "notvalid", firstname_invalid);
    toggle_class(document.getElementById("lastname-label"), "notvalid", lastname_invalid);
    toggle_class(document.getElementById("modality-label"), "notvalid", modality_invalid);
    toggle_class(document.getElementById("affiliation-label"), "notvalid", affiliation_invalid);
    toggle_class(document.getElementById("hcrw-label"), "notvalid", hcrw_invalid);

    if (level_invalid || presenting_invalid || email_invalid || firstname_invalid || lastname_invalid || modality_invalid || affiliation_invalid) {
        return true;
    }

    if (form.get("modality") === "online") {
        console.log(level_invalid + ", " + presenting_invalid + ", " + email_invalid + ", " + firstname_invalid + ", " + lastname_invalid + ", " + modality_invalid + ", " + affiliation_invalid);

        return false;
    } else {
        const countries = document.getElementById("countries");
        const cities = document.getElementById("cities");
        const country = form.get("departure-country");
        const city = form.get("departure-city");
        const transportation = form.get("transportation");

        const country_found = [...countries.options].find(opt => opt.value === country);
        const city_found = [...cities.options].find(opt => opt.value === city);

        const country_invalid = country === null || countries.options.length <= 0 || country === "" || !country_found;
        const city_invalid = city === null || city === "" || cities.options.length <= 0 || !city_found;
        const transportation_invalid = transportation === null;

        toggle_class(document.getElementById("departure-country-label"), "notvalid", country_invalid);
        toggle_class(document.getElementById("departure-city-label"), "notvalid", city_invalid);
        toggle_class(document.getElementById("transportation-label"), "notvalid", transportation_invalid);

        return country_invalid || city_invalid || transportation_invalid;
    }
}

function update_form() {
    var data = new FormData(document.getElementById("registration-form"))
    modality_toggle(data.get("modality") === "online");

    document.getElementById("submit").disabled = form_invalid(data);
}